// ========================================================
//
// CBP_3402_D_Bar_Codes
// 01-11-21
// Based on FBO_CBP_Line_Follower.ino
// FBO CBP 20-07-21
// Line following using Pixy2
//
// ========================================================

#include <Pixy2.h>
#include <Servo.h>                           

// Define pins ----------------------
#define ServoL_pin 13
#define ServoR_pin 12

// Global variables -----------------
Servo servoL;                                
Servo servoR;
float driveL;
float driveR;

// Globals for this task ------------
float error;
float lastError;
float deriv;
float Kp;
float Kd;
int fwd;

// PixyCam stuff -------------------
// This is the main Pixy object 
Pixy2 pixy;
uint8_t x0;
uint8_t y0;
uint16_t frameWidth;
uint16_t frameHeight;
uint8_t barCode;
uint8_t numBarCodes;

void setup(){
  Serial.begin(9600); 
  cprintf(">>> Initializing stuff ..................\n"); 
  cprintf(">>> Pausing 2 secs ......................\n");
  delay(2000);
  cprintf(">>> Starting  ...........................\n");
  servoL.attach(ServoL_pin);
  servoR.attach(ServoR_pin);

  pixy.init();
  // Change to line tracking program
  pixy.changeProg("line");
  pixy.getResolution();
  frameWidth = pixy.frameWidth;
  frameHeight = pixy.frameHeight;
  cprintf("frameWidth = %d, frameHeight = %d\n",
          frameWidth,frameHeight);  
  pixy.setLamp(1,0);

  // Tune the PID --------------------------------------
  Kp = 150.0; Kd = 100.0; fwd = 20; // OK

}

// This code repeats indefinitely
void loop() {

  pixy.line.getAllFeatures();

  x0 = pixy.line.vectors[0].m_x0;
  barCode = pixy.line.barcodes[0].m_code;
  numBarCodes = pixy.line.numBarcodes;
  
  cprintf("numBarCodes = %d, barCode = %d\n",numBarCodes,barCode);

  error = (float)x0 - (float)frameWidth/2.0;
  error = error / ((float)(frameWidth/2));

  deriv = error - lastError;
  lastError = error;
  
  driveL = fwd + Kp*error + Kd*deriv;
  driveR = fwd - Kp*error - Kd*deriv;

  driveServos(driveL,driveR);

  // Handle bar codes -----------------------------------
  if(numBarCodes > 0) {
    switch(barCode) {
      case 0:
        break;
      case 1:
        driveServos(30,0);
        break;
      case 2:
        driveServos(0,30);
        break;
    }
  }
  //delay(250);                       
}

void driveServos(float _driveL, float _driveR) {
  servoL.writeMicroseconds(1500 + (int)_driveL);
  servoR.writeMicroseconds(1500 - (int)_driveR);
}
