// =========================================================
// CBP_3402_D_Servo_Tweak
// From CBP_FBO Snake_Angle_Tweak Oct 2021.
// And CBP_2403_Servo_Tweak
// Zeros the servos after applying FWD drive
// =========================================================

#include <Servo.h>                           

// Define pins ---------------
#define ServoL_pin 13
#define ServoR_pin 12
#define EncoderL_pin 19
#define EncoderR_pin 20

// Define constants ----------
#define DEBUG true
#define wheelDiam 66.0
#define wheelRad (wheelDiam/2.0)
#define axleLen 104.0
#define dx ((PI*wheelDiam)/64.0)


// Global variables ----------
Servo myServos[2];
int offset;


void setup() {
  Serial.begin(9600);
  cprintf(">>> Initializing stuff ..................\n"); 

  
  cprintf(">>> Pausing 2 secs ......................\n");
  delay(2000);
  cprintf(">>> Starting  ...........................\n");

  cprintf(">>> Attaching Servos ....................\n");

  servoAttach(0);
  servoAttach(1);

  cprintf(">>> Servo Drive Cycle Starting ..........\n");

  // Forwards for 2 seconds
  driveMyServos(30,30);
  delay(2000);
  // Backwards for 2 seconds
  driveMyServos(-30,-30);
  delay(2000);
  // Stop
  driveMyServos(0,0);
  delay(2000);

  cprintf(">>> Servo Drive Cycle Complete ..........\n");
  cprintf(">>> Ready for you to do Tweaks ..........\n");

  myServos[0].detach();
  myServos[1].detach();
           
}

void servoAttach(int index) {
  switch(index) {
    case 0:
      myServos[0].attach(ServoL_pin);
      break;
    case 1:
      myServos[1].attach(ServoR_pin); 
      break;    
                             
  }
  
}

void loop(){  
  int index;
  if(Serial.available()) {
    char ch = (char)Serial.read();
    Serial.println(ch);
    if(isDigit(ch)) {
      index = (ch - '0');
      servoAttach(index);
      myServos[index].writeMicroseconds(1500);
      delay(15);       
      offset = 0;
      Serial.print("index = ");
      Serial.println(index);       
    }
    if(ch == 43) {
      offset += 5;
      myServos[index].writeMicroseconds(1500 + offset);
      delay(15);       
      Serial.print("Offset = ");
      Serial.println(offset);      
    }
    if(ch == 45) {
      offset -= 5;      
      myServos[index].writeMicroseconds(1500 + offset);
      delay(15);       
      Serial.print("Offset = ");
      Serial.println(offset);      
    }
     if(ch == 42) {     
      myServos[index].detach();
      Serial.print("Servo Detached ");      
    }

  }

  delay(500);
}

void driveMyServos(float _driveL, float _driveR) {
  myServos[0].writeMicroseconds(1500 + (int)_driveL);
  myServos[1].writeMicroseconds(1500 - (int)_driveR);
}
